@@ -2,6 +2,19 @@
 
 set -o errexit
 
+FORCE_GENERATE="${FORCE_GENERATE}"
+
+if [[ ${KOLLA_INSTALL_TYPE} == "binary" ]]; then
+    SITE_PACKAGES="/usr/lib/python2.7/site-packages"
+elif [[ ${KOLLA_INSTALL_TYPE} == "source" ]]; then
+    SITE_PACKAGES="/var/lib/kolla/venv/lib/python2.7/site-packages"
+fi
+
+if [[ ${KOLLA_INSTALL_TYPE} == "source" ]] && [[ ! -f ${SITE_PACKAGES}/openstack_dashboard/local/local_settings.py ]]; then
+    ln -s /etc/openstack-dashboard/local_settings \
+        ${SITE_PACKAGES}/openstack_dashboard/local/local_settings.py
+fi
+
 # Bootstrap and exit if KOLLA_BOOTSTRAP variable is set. This catches all cases
 # of the KOLLA_BOOTSTRAP variable being set, including empty.
 if [[ "${!KOLLA_BOOTSTRAP[@]}" ]]; then
@@ -13,14 +26,6 @@ if [[ "${!KOLLA_BOOTSTRAP[@]}" ]]; then
     exit 0
 fi
 
-FORCE_GENERATE="no"
-
-if [[ ${KOLLA_INSTALL_TYPE} == "binary" ]]; then
-    SITE_PACKAGES="/usr/lib/python2.7/site-packages"
-elif [[ ${KOLLA_INSTALL_TYPE} == "source" ]]; then
-    SITE_PACKAGES="/var/lib/kolla/venv/lib/python2.7/site-packages"
-fi
-
 function config_dashboard {
     ENABLE=$1
     SRC=$2
@@ -179,9 +184,9 @@ function config_trove_dashboard {
 }
 
 function config_watcher_dashboard {
-    for file in ${SITE_PACKAGES}/watcher_dashboard/enabled/_*[^__].py; do
+    for file in ${SITE_PACKAGES}/watcher_dashboard/local/enabled/_*[^__].py; do
         config_dashboard "${ENABLE_WATCHER}" \
-            "${SITE_PACKAGES}/watcher_dashboard/enabled/${file##*/}" \
+            "${SITE_PACKAGES}/watcher_dashboard/local/enabled/${file##*/}" \
             "${SITE_PACKAGES}/openstack_dashboard/local/enabled/${file##*/}"
     done
 
@@ -198,6 +203,14 @@ function config_zaqar_dashboard {
     done
 }
 
+function config_zun_dashboard {
+    for file in ${SITE_PACKAGES}/zun_ui/enabled/_*[^__].py; do
+        config_dashboard "${ENABLE_ZUN}" \
+            "${SITE_PACKAGES}/zun_ui/enabled/${file##*/}" \
+            "${SITE_PACKAGES}/openstack_dashboard/local/enabled/${file##*/}"
+    done
+}
+
 config_cloudkitty_dashboard
 config_designate_dashboard
 config_freezer_ui
@@ -216,6 +229,7 @@ config_tacker_dashboard
 config_trove_dashboard
 config_watcher_dashboard
 config_zaqar_dashboard
+config_zun_dashboard
 
 # NOTE(pbourke): httpd will not clean up after itself in some cases which
 # results in the container not being able to restart. (bug #1489676, 1557036)
@@ -230,7 +244,7 @@ fi
 # NOTE(jeffrey4l): The local_settings file affect django-compress
 # behavior, so re-generate the compressed javascript and css if it
 # is changed
-MD5SUM_TXT_PATH="/var/lib/kolla/.local_settings.md5sum.txt"
+MD5SUM_TXT_PATH="/tmp/.local_settings.md5sum.txt"
 if [[ ! -f ${MD5SUM_TXT_PATH} || $(md5sum -c --status ${MD5SUM_TXT_PATH};echo $?) != 0 || ${FORCE_GENERATE} == "yes" ]]; then
     md5sum /etc/openstack-dashboard/local_settings > ${MD5SUM_TXT_PATH}
     if [[ "${KOLLA_INSTALL_TYPE}" == "binary" ]]; then
@@ -247,6 +261,11 @@ fi
 if [[ ! -d "/var/log/kolla/horizon" ]]; then
     mkdir -p /var/log/kolla/horizon
 fi
+
 if [[ $(stat -c %a /var/log/kolla/horizon) != "755" ]]; then
     chmod 755 /var/log/kolla/horizon
 fi
+
+if [[ -f ${SITE_PACKAGES}/openstack_dashboard/local/.secret_key_store ]] && [[ $(stat -c %U ${SITE_PACKAGES}/openstack_dashboard/local/.secret_key_store) != "horizon" ]]; then
+    chown horizon ${SITE_PACKAGES}/openstack_dashboard/local/.secret_key_store
+fi
