@@ -6,11 +6,8 @@ set -o errexit
 # Enable unbuffered output for Ansible in Jenkins.
 export PYTHONUNBUFFERED=1
 
-source /etc/nodepool/provider
-
-NODEPOOL_MIRROR_HOST=${NODEPOOL_MIRROR_HOST:-mirror.$NODEPOOL_REGION.$NODEPOOL_CLOUD.openstack.org}
-NODEPOOL_MIRROR_HOST=$(echo $NODEPOOL_MIRROR_HOST|tr '[:upper:]' '[:lower:]')
-NODEPOOL_PYPI_MIRROR=${NODEPOOL_PYPI_MIRROR:-http://$NODEPOOL_MIRROR_HOST/pypi/simple}
+. /etc/nodepool/provider
+. /etc/ci/mirror_info.sh
 
 # Just for mandre :)
 if [[ ! -f /etc/sudoers.d/jenkins ]]; then
@@ -18,12 +15,8 @@ if [[ ! -f /etc/sudoers.d/jenkins ]]; then
 fi
 
 function setup_config {
-    # generate the config
-    tox -e genconfig
-    # Copy configs
-    sudo cp -a etc/kolla /etc/
-    # Generate passwords
-    sudo tools/generate_passwords.py
+
+    sudo mkdir -p /etc/kolla
 
     # Use Infra provided pypi.
     # Wheel package mirror may be not compatible. So do not enable it.
@@ -34,29 +27,63 @@ timeout = 60
 index-url = $NODEPOOL_PYPI_MIRROR
 trusted-host = $NODEPOOL_MIRROR_HOST
 EOF
-    echo "RUN echo $(base64 -w0 ${PIP_CONF}) | base64 -d > /etc/pip.conf" | sudo tee /etc/kolla/header
-    rm ${PIP_CONF}
-    sed -i 's|^#include_header.*|include_header = /etc/kolla/header|' /etc/kolla/kolla-build.conf
+    TEMPLATE_OVERRIDES=$(mktemp)
 
-    # NOTE(Jeffrey4l): use different a docker namespace name in case it pull image from hub.docker.io when deplying
-    sed -i 's|^#namespace.*|namespace = lokolla|' /etc/kolla/kolla-build.conf
+    cat <<EOF | tee "${TEMPLATE_OVERRIDES}"
+{% extends parent_template %}
 
-    if [[ "${DISTRO}" == "Debian" ]]; then
-        # Infra does not sign thier mirrors so we ignore gpg signing in the gate
-        echo "RUN echo 'APT::Get::AllowUnauthenticated \"true\";' > /etc/apt/apt.conf" | sudo tee -a /etc/kolla/header
+{% block base_header %}
 
-        # Optimize the repos to take advantage of the Infra provided mirrors for Ubuntu
-        sed -i 's|^#apt_sources_list.*|apt_sources_list = /etc/kolla/sources.list|' /etc/kolla/kolla-build.conf
-        sudo cp /etc/apt/sources.list /etc/kolla/sources.list
-        sudo cat /etc/apt/sources.list.available.d/ubuntu-cloud-archive.list | sudo tee -a /etc/kolla/sources.list
-        # Append non-infra provided repos to list
-        cat << EOF | sudo tee -a /etc/kolla/sources.list
-deb http://nyc2.mirrors.digitalocean.com/mariadb/repo/10.0/ubuntu trusty main
-deb http://repo.percona.com/apt trusty main
-deb http://packages.elastic.co/elasticsearch/2.x/debian stable main
-deb http://packages.elastic.co/kibana/4.4/debian stable main
+RUN echo $(base64 -w0 "${PIP_CONF}") | base64 -d > /etc/pip.conf
+
+{% if base_distro == 'ubuntu' %}
+
+RUN echo 'APT::Get::AllowUnauthenticated "true";' > /etc/apt/apt.conf.d/99allow-unauthenticated
+
+{% endif %}
+{% endblock %}
+
+{% block base_footer %}
+{% if base_distro == "centos" %}
+
+RUN sed -i -e "/^mirrorlist/d" \
+        -e "s|^#baseurl=http://mirror.centos.org|baseurl=http://$NODEPOOL_MIRROR_HOST|" \
+        /etc/yum.repos.d/CentOS-Base.repo \
+    && sed -i -e "/^mirrorlist/d" \
+        -e "s|^#baseurl=http://download.fedoraproject.org/pub|baseurl=http://$NODEPOOL_MIRROR_HOST|" \
+        /etc/yum.repos.d/epel.repo \
+    && sed -i -e "s|^baseurl=http://mirror.centos.org|baseurl=http://$NODEPOOL_MIRROR_HOST|" \
+        /etc/yum.repos.d/CentOS-Ceph-Jewel.repo
+
+{% elif base_distro == "oraclelinux" %}
+
+RUN sed -i -e "/^mirrorlist/d" \
+        -e "s|^#baseurl=http://download.fedoraproject.org/pub|baseurl=http://$NODEPOOL_MIRROR_HOST|" \
+        /etc/yum.repos.d/epel.repo \
+    && sed -i -e "s|^baseurl=http://mirror.centos.org|baseurl=http://$NODEPOOL_MIRROR_HOST|" \
+        /etc/yum.repos.d/CentOS-Ceph-Jewel.repo
+
+{% elif base_distro == "ubuntu" %}
+
+RUN sed -i -e "s|http://archive.ubuntu.com|http://$NODEPOOL_MIRROR_HOST|" \
+        -e "s|http://ubuntu-cloud.archive.canonical.com/ubuntu|http://$NODEPOOL_MIRROR_HOST/ubuntu-cloud-archive|" \
+        /etc/apt/sources.list \
+    && apt-get update
+
+{% endif %}
+{% endblock %}
 EOF
-    fi
+
+cat <<EOF | sudo tee /etc/kolla/kolla-build.conf
+[DEFAULT]
+# NOTE(Jeffrey4l): use different a docker namespace name in case it pull image from hub.docker.io when deplying
+namespace = lokolla
+template_override = ${TEMPLATE_OVERRIDES}
+registry = 127.0.0.1:4000
+push = true
+logs_dir = /tmp/logs/build
+EOF
+
 }
 
 function detect_distro {
@@ -88,6 +115,7 @@ function setup_ssh {
 
     # From now on use the new IdentityFile for connecting to other hosts
     echo "IdentityFile /home/jenkins/.ssh/kolla" >> /home/jenkins/.ssh/config
+    chmod 600 /home/jenkins/.ssh/config
 }
 
 function setup_inventory {
@@ -100,6 +128,7 @@ function setup_inventory {
         # wordround fix for the rabbitmq failed when deploy on CentOS in the CI
         # gate. the ideal fix should set the hostname in setup_gate.sh script.
         # But it do not work as expect with unknown reason
+        ssh-keyscan "${ip}" >> ~/.ssh/known_hosts
         echo -e "${ip}\tnode${counter}" >> /tmp/hosts
         echo -e "${ip}\t$(ssh ${ip} hostname)" >> /tmp/hosts
         echo "node${counter}" >> ${RAW_INVENTORY}
@@ -115,24 +144,25 @@ function setup_ansible {
     mkdir /tmp/kolla
 
     # TODO(SamYaple): Move to virtualenv
-    sudo -H pip install -U "ansible>=2" "docker-py>=1.6.0" "python-openstackclient" "python-neutronclient"
+    sudo -H pip install -U "ansible>=2" "docker>=2.0.0" "python-openstackclient" "python-neutronclient"
     detect_distro
 
     setup_inventory
 
-    # Record the running state of the environment as seen by the setup module
-    ansible all -i ${RAW_INVENTORY} -m setup > /tmp/logs/ansible/initial-setup
-
     sudo -H pip install ara
     sudo mkdir /etc/ansible
     sudo tee /etc/ansible/ansible.cfg<<EOF
 [defaults]
-callback_plugins = /usr/lib/python2.7/site-packages/ara/callback:\$VIRTUAL_ENV/lib/python2.7/site-packages/ara/callback:/usr/local/lib/python2.7/dist-packages/ara/callback
+callback_plugins = /usr/lib/python2.7/site-packages/ara/plugins/callbacks:\$VIRTUAL_ENV/lib/python2.7/site-packages/ara/plugins/callbacks
+host_key_checking = False
 EOF
+
+    # Record the running state of the environment as seen by the setup module
+    ansible all -i ${RAW_INVENTORY} -m setup > /tmp/logs/ansible/initial-setup
 }
 
 function setup_node {
-    ansible-playbook -i ${RAW_INVENTORY} tools/setup_nodes.yml
+    ansible-playbook -i ${RAW_INVENTORY} tools/playbook-setup-nodes.yml
 }
 
 function setup_logging {
@@ -145,10 +175,17 @@ function setup_logging {
     mkdir -p /tmp/logs/{ansible,build,kolla,kolla_configs,system_logs}
 }
 
+function setup_registry {
+    sudo mkdir /opt/kolla_registry
+    sudo chmod -R 644 /opt/kolla_registry
+    docker run -d -p 4000:5000 --restart=always -v /opt/kolla_registry/:/var/lib/registry --name registry registry:2
+}
+
 setup_logging
 tools/dump_info.sh
 setup_workaround_broken_nodepool
 setup_ssh
 setup_ansible
 setup_node
+setup_registry
 setup_config
